/* ``The contents of this file are subject to the Erlang Public License,
 * Version 1.0, (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.erlang.org/EPL1_0.txt
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific language governing rights and limitations
 * under the License.
 * 
 * The Original Code is Erlang-4.7.3, December, 1998.
 * 
 * The Initial Developer of the Original Code is Ericsson Telecom
 * AB. Portions created by Ericsson are Copyright (C), 1998, Ericsson
 * Telecom AB. All Rights Reserved.
 * 
 * Contributor(s): ______________________________________.''
 */
/* Created:  24 Jul 1995 by tobbe@erix.ericsson.se
 *
 * Function: Example of a C server, accepting connection attempts,
 *           and calls from Erlang.
 *
 * RCS: $Id: igsock_server.c,v 1.3 1996/04/09 13:03:57 tobbe Exp tobbe $
 */
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <sys/time.h>
#include "ig.h"

#ifdef SUNOS4
extern int fprintf();
extern void perror();
extern int select();
extern void bzero();
#endif

#ifdef SOLARIS
extern void *memset();
#endif

extern int close();

extern int igsock_connect();
extern int igsock_listen();
extern int igsock_accept();
extern int ig_read();

static void syserr(char *msg) 
{ 
  perror(msg); 
  exit(-1);
} /* syserr */

#define MY_TIMEOUT 60

extern int ig_fun_in;
extern int ig_dispatch();

int main(int argc, char *argv[])
{
  int rc,len,fd,port;
  int efd=IG_NO_FD,lfd=IG_NO_FD;
  struct timeval tv,*tvp;
  fd_set readmask;


  if (argc == 2) 
    port = atoi(argv[argc-1]);
  else {
    fprintf(stderr,"Usuage: %s <PortNo>\n",argv[0]);
    exit(-1);
  }

  erl_init((Erl_Heap *) NULL, 0);

  if ((lfd = igsock_listen( port )) < 0) {
    fprintf(stderr, "ERROR 1 !\n");
    exit(-1);
  }

  tv.tv_sec = MY_TIMEOUT;
  tv.tv_usec = 0;
  tvp = &tv;

  while (1) {

    FD_ZERO( &readmask );

    if (lfd != IG_NO_FD) 
      FD_SET( lfd , &readmask );     /* Waiting for a remote connect */ 

    if (efd != IG_NO_FD) 
      FD_SET( efd , &readmask );     /* Waiting for Erlang requests */ 

    if ((rc = select (FD_SETSIZE, &readmask, (fd_set *)0, (fd_set *)0,
                      (struct timeval *) tvp  )) < 0 ) 
      syserr("select");

    if (rc == 0) { 
      fprintf(stderr,"TIMED OUT !!\n");
      exit(-1);
    }

    if ((efd != IG_NO_FD) && FD_ISSET( efd , &readmask )) { 
      if ((len = ig_read( ig_fun_in )) > 0)
	ig_dispatch();
      else {
	fprintf(stderr,"Connection broken !\n");
	exit(-1);
      }
    }

    if ((lfd != IG_NO_FD) && FD_ISSET( lfd , &readmask )) { 
      if ((fd = igsock_accept( lfd )) < 0)
	fprintf(stderr,"ERROR 2 !\n");
      else {
	close( lfd );
	lfd = IG_NO_FD;
	tvp = (struct timeval *) NULL;
	efd = fd;
      }
    }
  } /* while */

  fprintf(stderr,"Ready !\n");

}


